package one;

import de.upb.tools.sdm.*;
import java.util.*;
import de.upb.tools.fca.*;
import org.apache.log4j.Logger;

public class Container
{
	private static Logger logger = Logger.getLogger(Container.class);
   /**
    * <pre>
    *            0..1   processContainer   0..n 
    * Container -------------------------------- Process
    *            container            processes 
    * </pre>
    */
   private FHashSet processes;

   public boolean addToProcesses(Process value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.processes == null)
         {
            this.processes = new FHashSet (); // or FTreeSet () or FLinkedList ()
         }
         
         changed = this.processes.add (value);
         if (changed)
         {
            value.setContainer (this);
         }
         
      }
      return changed;
   }

   public boolean hasInProcesses(Process value)
   {
      return ((this.processes != null) &&
              (value != null) &&
              this.processes.contains (value));
   }

   public Iterator iteratorOfProcesses()
   {
      return ((this.processes == null)
              ? FEmptyIterator.get ()
              : this.processes.iterator ());
   }

   public void removeAllFromProcesses()
   {
      Process tmpValue;
      Iterator iter = this.iteratorOfProcesses ();
      while (iter.hasNext ())
      {
         tmpValue = (Process) iter.next ();
         this.removeFromProcesses (tmpValue);
      }
   }

   public boolean removeFromProcesses(Process value)
   {
      boolean changed = false;
      if ((this.processes != null) && (value != null))
      {
         
         changed = this.processes.remove (value);
         if (changed)
         {
            value.setContainer (null);
         }
         
      }
      return changed;
   }

   public int sizeOfProcesses()
   {
      return ((this.processes == null)
              ? 0
              : this.processes.size ());
   }

   /**
    * <pre>
    *            0..1   resourceContainer   0..n 
    * Container --------------------------------- Resource
    *            container             resources 
    * </pre>
    */
   private FHashSet resources;

   public boolean addToResources(Resource value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.resources == null)
         {
            this.resources = new FHashSet (); // or FTreeSet () or FLinkedList ()
         }
         
         changed = this.resources.add (value);
         if (changed)
         {
            value.setContainer (this);
         }
         
      }
      return changed;
   }

   public boolean hasInResources(Resource value)
   {
      return ((this.resources != null) &&
              (value != null) &&
              this.resources.contains (value));
   }

   public Iterator iteratorOfResources()
   {
      return ((this.resources == null)
              ? FEmptyIterator.get ()
              : this.resources.iterator ());
   }

   public void removeAllFromResources()
   {
      Resource tmpValue;
      Iterator iter = this.iteratorOfResources ();
      while (iter.hasNext ())
      {
         tmpValue = (Resource) iter.next ();
         this.removeFromResources (tmpValue);
      }
   }

   public boolean removeFromResources(Resource value)
   {
      boolean changed = false;
      if ((this.resources != null) && (value != null))
      {
         
         changed = this.resources.remove (value);
         if (changed)
         {
            value.setContainer (null);
         }
         
      }
      return changed;
   }

   public int sizeOfResources()
   {
      return ((this.resources == null)
              ? 0
              : this.resources.size ());
   }

   public void blockedRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterP1RequestsR = null ;
      Iterator fujaba__IterThisProcessesP1 = null ;
      Process p1 = null ;
      Process p2 = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p1: Process
         fujaba__IterThisProcessesP1 = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP1.hasNext () )
         {
            try
            {
               p1 = (Process) fujaba__IterThisProcessesP1.next () ;
                
               // bind r: Resource
               fujaba__IterP1RequestsR = p1.iteratorOfRequests () ;
               while ( !(fujaba__Success) && fujaba__IterP1RequestsR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterP1RequestsR.next () ;
                      
                     // bind p2: Process
                     p2 = r.getHeldBy () ;
                     JavaSDM.ensure ( p2 != null ) ;
                      
                     // check isomorphic binding
                     JavaSDM.ensure ( !(p1.equals (p2)) ) ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("blockedRule: pm: " + System.nanoTime());
}
                     // create link
                     r.addToBlocks (p1);
                      
                     fujaba__Success = true ;
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void giveRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterP1ReleasesR = null ;
      Iterator fujaba__IterThisProcessesP1 = null ;
      Process p1 = null ;
      Process p2 = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p1: Process
         fujaba__IterThisProcessesP1 = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP1.hasNext () )
         {
            try
            {
               p1 = (Process) fujaba__IterThisProcessesP1.next () ;
                
               // bind p2: Process
               p2 = p1.getNext () ;
               JavaSDM.ensure ( p2 != null ) ;
                
               // check isomorphic binding
               JavaSDM.ensure ( !(p1.equals (p2)) ) ;
                
               // bind r: Resource
               fujaba__IterP1ReleasesR = p1.iteratorOfReleases () ;
               while ( !(fujaba__Success) && fujaba__IterP1ReleasesR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterP1ReleasesR.next () ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("giveRule: pm: " + System.nanoTime());
}
                     //  delete link
                     r.setReleasedBy (null);
                      
                     // create link
                     r.setToken (p2);
                      
                     fujaba__Success = true ;
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void giveRulePar()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterP1ReleasesR = null ;
      Iterator fujaba__IterThisProcessesP1 = null ;
      Process p1 = null ;
      Process p2 = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p1: Process
         fujaba__IterThisProcessesP1 = this.iteratorOfProcesses () ;
         while ( fujaba__IterThisProcessesP1.hasNext () )
         {
            try
            {
               p1 = (Process) fujaba__IterThisProcessesP1.next () ;
                
               // bind p2: Process
               p2 = p1.getNext () ;
               JavaSDM.ensure ( p2 != null ) ;
                
               // check isomorphic binding
               JavaSDM.ensure ( !(p1.equals (p2)) ) ;
                
               // bind r: Resource
               fujaba__IterP1ReleasesR = p1.iteratorOfReleases () ;
               while ( fujaba__IterP1ReleasesR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterP1ReleasesR.next () ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("giveRulePar: pm: " + System.nanoTime());
}
                     //  delete link
                     r.setReleasedBy (null);
                      
                     // create link
                     r.setToken (p2);
                      
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void ignoreRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPBlockedByR = null ;
      Iterator fujaba__IterThisProcessesP = null ;
      Process p = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP.hasNext () )
         {
            try
            {
               p = (Process) fujaba__IterThisProcessesP.next () ;
                
               // check To-Many-Link 'holds' between p and rn
               JavaSDM.ensure ( p.sizeOfHolds () == 0 ) ;
                
               // bind r: Resource
               fujaba__IterPBlockedByR = p.iteratorOfBlockedBy () ;
               while ( !(fujaba__Success) && fujaba__IterPBlockedByR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterPBlockedByR.next () ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("ignoreRule: pm: " + System.nanoTime());
}
                     //  delete link
                     r.removeFromBlocks (p);
                      
                     fujaba__Success = true ;
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void killRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterThisProcessesP1 = null ;
      Process p = null ;
      Process p1 = null ;
      Process p2 = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p1: Process
         fujaba__IterThisProcessesP1 = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP1.hasNext () )
         {
            try
            {
               p1 = (Process) fujaba__IterThisProcessesP1.next () ;
                
               // bind p: Process
               p = p1.getNext () ;
               JavaSDM.ensure ( p != null ) ;
                
               // check isomorphic binding
               JavaSDM.ensure ( !(p1.equals (p)) ) ;
                
               // bind p2: Process
               p2 = p.getNext () ;
               JavaSDM.ensure ( p2 != null ) ;
                
               // check isomorphic binding
               JavaSDM.ensure ( !(p1.equals (p2)) && !(p.equals (p2)) ) ;
                
if(logger.isDebugEnabled()) {
	logger.debug("killRule: pm: " + System.nanoTime());
}
               // delete object
               p.removeYou ();
                
               // create link
               p1.setNext (p2);
                
               fujaba__Success = true ;
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void mountRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterThisProcessesP = null ;
      Process p = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP.hasNext () )
         {
            try
            {
               p = (Process) fujaba__IterThisProcessesP.next () ;
                
if(logger.isDebugEnabled()) {
	logger.debug("mountRule: pm: " + System.nanoTime());
}
               // create object
               r = new Resource ( ) ;
                
               // create link
               p.addToTokens (r);
                
               // create link
               this.addToResources (r);
                
               fujaba__Success = true ;
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void newRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterThisProcessesP1 = null ;
      Process p = null ;
      Process p1 = null ;
      Process p2 = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p1: Process
         fujaba__IterThisProcessesP1 = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP1.hasNext () )
         {
            try
            {
               p1 = (Process) fujaba__IterThisProcessesP1.next () ;
                
               // bind p2: Process
               p2 = p1.getNext () ;
               JavaSDM.ensure ( p2 != null ) ;
                
               // check isomorphic binding
               JavaSDM.ensure ( !(p1.equals (p2)) ) ;
                
if(logger.isDebugEnabled()) {
	logger.debug("newRule: pm: " + System.nanoTime());
}
               //  delete link
               p1.setNext (null);
                
               // create object
               p = new Process ( ) ;
                
               // create link
               p2.setPrevious (p);
                
               // create link
               p1.setNext (p);
                
               // create link
               this.addToProcesses (p);
                
               fujaba__Success = true ;
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void passRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterP1TokensR = null ;
      Iterator fujaba__IterThisProcessesP1 = null ;
      Process p1 = null ;
      Process p2 = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p1: Process
         fujaba__IterThisProcessesP1 = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP1.hasNext () )
         {
            try
            {
               p1 = (Process) fujaba__IterThisProcessesP1.next () ;
                
               // bind p2: Process
               p2 = p1.getNext () ;
               JavaSDM.ensure ( p2 != null ) ;
                
               // check isomorphic binding
               JavaSDM.ensure ( !(p1.equals (p2)) ) ;
                
               // bind r: Resource
               fujaba__IterP1TokensR = p1.iteratorOfTokens () ;
               while ( !(fujaba__Success) && fujaba__IterP1TokensR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterP1TokensR.next () ;
                      
                     // check To-Many-Link 'requests' between p1 and r
                     JavaSDM.ensure ( !(p1.hasInRequests (r)) ) ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("passRule: pm: " + System.nanoTime());
}
                     //  delete link
                     r.setToken (null);
                      
                     // create link
                     r.setToken (p2);
                      
                     fujaba__Success = true ;
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void releaseRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPHoldsR = null ;
      Iterator fujaba__IterThisProcessesP = null ;
      Process p = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP.hasNext () )
         {
            try
            {
               p = (Process) fujaba__IterThisProcessesP.next () ;
                
               // check To-Many-Link 'requests' between p and rn
               JavaSDM.ensure ( p.sizeOfRequests () == 0 ) ;
                
               // bind r: Resource
               fujaba__IterPHoldsR = p.iteratorOfHolds () ;
               while ( !(fujaba__Success) && fujaba__IterPHoldsR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterPHoldsR.next () ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("releaseRule: pm: " + System.nanoTime());
}
                     //  delete link
                     r.setHeldBy (null);
                      
                     // create link
                     r.setReleasedBy (p);
                      
                     fujaba__Success = true ;
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void releaseRulePar()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPHoldsR = null ;
      Iterator fujaba__IterThisProcessesP = null ;
      Process p = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
         while ( fujaba__IterThisProcessesP.hasNext () )
         {
            try
            {
               p = (Process) fujaba__IterThisProcessesP.next () ;
                
               // check To-Many-Link 'requests' between p and rn
               JavaSDM.ensure ( p.sizeOfRequests () == 0 ) ;
                
               // bind r: Resource
               fujaba__IterPHoldsR = p.iteratorOfHolds () ;
               while ( fujaba__IterPHoldsR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterPHoldsR.next () ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("releaseRulePar: pm: " + System.nanoTime());
}
                     //  delete link
                     r.setHeldBy (null);
                      
                     // create link
                     r.setReleasedBy (p);
                      
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void releaseStarRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterP1RequestsR1 = null ;
      Iterator fujaba__IterP2HoldsR2 = null ;
      Iterator fujaba__IterThisProcessesP1 = null ;
      Process p1 = null ;
      Process p2 = null ;
      Resource r1 = null ;
      Resource r2 = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p1: Process
         fujaba__IterThisProcessesP1 = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP1.hasNext () )
         {
            try
            {
               p1 = (Process) fujaba__IterThisProcessesP1.next () ;
                
               // bind r1: Resource
               fujaba__IterP1RequestsR1 = p1.iteratorOfRequests () ;
               while ( !(fujaba__Success) && fujaba__IterP1RequestsR1.hasNext () )
               {
                  try
                  {
                     r1 = (Resource) fujaba__IterP1RequestsR1.next () ;
                      
                     // bind p2: Process
                     p2 = r1.getHeldBy () ;
                     JavaSDM.ensure ( p2 != null ) ;
                      
                     // check isomorphic binding
                     JavaSDM.ensure ( !(p1.equals (p2)) ) ;
                      
                     // bind r2: Resource
                     fujaba__IterP2HoldsR2 = p2.iteratorOfHolds () ;
                     while ( !(fujaba__Success) && fujaba__IterP2HoldsR2.hasNext () )
                     {
                        try
                        {
                           r2 = (Resource) fujaba__IterP2HoldsR2.next () ;
                            
                           // check isomorphic binding
                           JavaSDM.ensure ( !(r1.equals (r2)) ) ;
                            
if(logger.isDebugEnabled()) {
	logger.debug("releaseStarRule: pm: " + System.nanoTime());
}
                           //  delete link
                           r1.setHeldBy (null);
                            
                           // create link
                           r1.setReleasedBy (p2);
                            
                           fujaba__Success = true ;
                        }
                        catch ( JavaSDMException fujaba__InternalException )
                        {
                        }
                     }
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void requestRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPRequestsRn = null ;
      Iterator fujaba__IterThisProcessesP = null ;
      Iterator fujaba__IterThisResourcesR = null ;
      Process p = null ;
      Resource r = null ;
      Resource rn = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind r: Resource
         fujaba__IterThisResourcesR = this.iteratorOfResources () ;
         while ( !(fujaba__Success) && fujaba__IterThisResourcesR.hasNext () )
         {
            try
            {
               r = (Resource) fujaba__IterThisResourcesR.next () ;
                
               // bind p: Process
               fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
               while ( !(fujaba__Success) && fujaba__IterThisProcessesP.hasNext () )
               {
                  try
                  {
                     p = (Process) fujaba__IterThisProcessesP.next () ;
                      
                     // check To-Many-Link 'requests' between p and r
                     JavaSDM.ensure ( !(p.hasInRequests (r)) ) ;
                      
                     // check To-One-Link 'heldBy' between r and p
                     JavaSDM.ensure ( (r == null) || ((r.getHeldBy () == null) || !(r.getHeldBy ().equals (p))) ) ;
                      
                     // check negative application condition
                     try
                     {
                        fujaba__Success = false ;
                         
                        // bind rn: Resource
                        fujaba__IterPRequestsRn = p.iteratorOfRequests () ;
                        while ( !(fujaba__Success) && fujaba__IterPRequestsRn.hasNext () )
                        {
                           try
                           {
                              rn = (Resource) fujaba__IterPRequestsRn.next () ;
                               
                              // check isomorphic binding
                              JavaSDM.ensure ( !(r.equals (rn)) ) ;
                               
                              fujaba__Success = true ;
                           }
                           catch ( JavaSDMException fujaba__InternalException )
                           {
                           }
                        }
                     }
                     catch ( JavaSDMException fujaba__InternalException )
                     {
                        fujaba__Success = false ;
                     }
                      
                     fujaba__Success = !(fujaba__Success);
                     JavaSDM.ensure ( fujaba__Success ) ;
                     fujaba__Success = false;
                     // end of negative application condition
                      
if(logger.isDebugEnabled()) {
	logger.debug("requestRule: pm: " + System.nanoTime());
}
                     // create link
                     p.addToRequests (r);
                      
                     fujaba__Success = true ;
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void requestSimpleRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPTokensR = null ;
      Iterator fujaba__IterThisProcessesP = null ;
      Process p = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP.hasNext () )
         {
            try
            {
               p = (Process) fujaba__IterThisProcessesP.next () ;
                
               // bind r: Resource
               fujaba__IterPTokensR = p.iteratorOfTokens () ;
               while ( !(fujaba__Success) && fujaba__IterPTokensR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterPTokensR.next () ;
                      
                     // check To-Many-Link 'requests' between p and r
                     JavaSDM.ensure ( !(p.hasInRequests (r)) ) ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("requestSimpleRule: pm: " + System.nanoTime());
}
                     // create link
                     p.addToRequests (r);
                      
                     fujaba__Success = true ;
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void requestSimpleRulePar()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPTokensR = null ;
      Iterator fujaba__IterThisProcessesP = null ;
      Process p = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
         while ( fujaba__IterThisProcessesP.hasNext () )
         {
            try
            {
               p = (Process) fujaba__IterThisProcessesP.next () ;
                
               // bind r: Resource
               fujaba__IterPTokensR = p.iteratorOfTokens () ;
               while ( fujaba__IterPTokensR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterPTokensR.next () ;
                      
                     // check To-Many-Link 'requests' between p and r
                     JavaSDM.ensure ( !(p.hasInRequests (r)) ) ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("requestSimpleRulePar: pm: " + System.nanoTime());
}
                     // create link
                     p.addToRequests (r);
                      
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void requestStarRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterP1HoldsR1 = null ;
      Iterator fujaba__IterP2HoldsR2 = null ;
      Iterator fujaba__IterThisProcessesP1 = null ;
      Process p1 = null ;
      Process p2 = null ;
      Resource r1 = null ;
      Resource r2 = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p1: Process
         fujaba__IterThisProcessesP1 = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP1.hasNext () )
         {
            try
            {
               p1 = (Process) fujaba__IterThisProcessesP1.next () ;
                
               // bind p2: Process
               p2 = p1.getPrevious () ;
               JavaSDM.ensure ( p2 != null ) ;
                
               // check isomorphic binding
               JavaSDM.ensure ( !(p1.equals (p2)) ) ;
                
               // bind r1: Resource
               fujaba__IterP1HoldsR1 = p1.iteratorOfHolds () ;
               while ( !(fujaba__Success) && fujaba__IterP1HoldsR1.hasNext () )
               {
                  try
                  {
                     r1 = (Resource) fujaba__IterP1HoldsR1.next () ;
                      
                     // bind r2: Resource
                     fujaba__IterP2HoldsR2 = p2.iteratorOfHolds () ;
                     while ( !(fujaba__Success) && fujaba__IterP2HoldsR2.hasNext () )
                     {
                        try
                        {
                           r2 = (Resource) fujaba__IterP2HoldsR2.next () ;
                            
                           // check isomorphic binding
                           JavaSDM.ensure ( !(r1.equals (r2)) ) ;
                            
                           // check To-Many-Link 'requests' between p1 and r2
                           JavaSDM.ensure ( !(p1.hasInRequests (r2)) ) ;
                            
if(logger.isDebugEnabled()) {
	logger.debug("requestStarRule: pm: " + System.nanoTime());
}
                           // create link
                           p1.addToRequests (r2);
                            
                           fujaba__Success = true ;
                        }
                        catch ( JavaSDMException fujaba__InternalException )
                        {
                        }
                     }
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void takeRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPRequestsR = null ;
      Iterator fujaba__IterThisProcessesP = null ;
      Process p = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP.hasNext () )
         {
            try
            {
               p = (Process) fujaba__IterThisProcessesP.next () ;
                
               // bind r: Resource
               fujaba__IterPRequestsR = p.iteratorOfRequests () ;
               while ( !(fujaba__Success) && fujaba__IterPRequestsR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterPRequestsR.next () ;
                      
                     // check To-One-Link 'token' between r and p
                     JavaSDM.ensure ( (r.getToken () != null) && r.getToken ().equals (p) ) ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("takeRule: pm: " + System.nanoTime());
}
                     //  delete link
                     r.setToken (null);
                      
                     //  delete link
                     p.removeFromRequests (r);
                      
                     // create link
                     r.setHeldBy (p);
                      
                     fujaba__Success = true ;
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public Resource takeRulePP()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPRequestsR = null ;
      Iterator fujaba__IterThisProcessesP = null ;
      Process p = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP.hasNext () )
         {
            try
            {
               p = (Process) fujaba__IterThisProcessesP.next () ;
                
               // bind r: Resource
               fujaba__IterPRequestsR = p.iteratorOfRequests () ;
               while ( !(fujaba__Success) && fujaba__IterPRequestsR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterPRequestsR.next () ;
                      
                     // check To-One-Link 'token' between r and p
                     JavaSDM.ensure ( (r.getToken () != null) && r.getToken ().equals (p) ) ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("takeRulePP: pm: " + System.nanoTime());
}
                     //  delete link
                     p.removeFromRequests (r);
                      
                     //  delete link
                     r.setToken (null);
                      
                     // create link
                     r.setHeldBy (p);
                      
                     fujaba__Success = true ;
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return (r) ;
   }


   public void takeRulePar()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPRequestsR = null ;
      Iterator fujaba__IterThisProcessesP = null ;
      Process p = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
         while ( fujaba__IterThisProcessesP.hasNext () )
         {
            try
            {
               p = (Process) fujaba__IterThisProcessesP.next () ;
                
               // bind r: Resource
               fujaba__IterPRequestsR = p.iteratorOfRequests () ;
               while ( fujaba__IterPRequestsR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterPRequestsR.next () ;
                      
                     // check To-One-Link 'token' between r and p
                     JavaSDM.ensure ( (r.getToken () != null) && r.getToken ().equals (p) ) ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("takeRulePar: pm: " + System.nanoTime());
}
                     //  delete link
                     p.removeFromRequests (r);
                      
                     //  delete link
                     r.setToken (null);
                      
                     // create link
                     r.setHeldBy (p);
                      
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void unlockRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPHoldsR = null ;
      Iterator fujaba__IterThisProcessesP = null ;
      Process p = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP.hasNext () )
         {
            try
            {
               p = (Process) fujaba__IterThisProcessesP.next () ;
                
               // bind r: Resource
               fujaba__IterPHoldsR = p.iteratorOfHolds () ;
               while ( !(fujaba__Success) && fujaba__IterPHoldsR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterPHoldsR.next () ;
                      
                     // check To-Many-Link 'blocks' between r and p
                     JavaSDM.ensure ( r.hasInBlocks (p) ) ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("unlockRule: pm: " + System.nanoTime());
}
                     //  delete link
                     r.setHeldBy (null);
                      
                     //  delete link
                     r.removeFromBlocks (p);
                      
                     // create link
                     r.setReleasedBy (p);
                      
                     fujaba__Success = true ;
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void unmountRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPTokensR = null ;
      Iterator fujaba__IterThisProcessesP = null ;
      Process p = null ;
      Resource r = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         fujaba__IterThisProcessesP = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP.hasNext () )
         {
            try
            {
               p = (Process) fujaba__IterThisProcessesP.next () ;
                
               // bind r: Resource
               fujaba__IterPTokensR = p.iteratorOfTokens () ;
               while ( !(fujaba__Success) && fujaba__IterPTokensR.hasNext () )
               {
                  try
                  {
                     r = (Resource) fujaba__IterPTokensR.next () ;
                      
if(logger.isDebugEnabled()) {
	logger.debug("unmountRule: pm: " + System.nanoTime());
}
                     // delete object
                     r.removeYou ();
                      
                     fujaba__Success = true ;
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void waitingRule()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterP1BlockedByR2 = null ;
      Iterator fujaba__IterP1HoldsR1 = null ;
      Iterator fujaba__IterR1RequestedByP2 = null ;
      Iterator fujaba__IterThisProcessesP1 = null ;
      Process p1 = null ;
      Process p2 = null ;
      Resource r1 = null ;
      Resource r2 = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p1: Process
         fujaba__IterThisProcessesP1 = this.iteratorOfProcesses () ;
         while ( !(fujaba__Success) && fujaba__IterThisProcessesP1.hasNext () )
         {
            try
            {
               p1 = (Process) fujaba__IterThisProcessesP1.next () ;
                
               // bind r1: Resource
               fujaba__IterP1HoldsR1 = p1.iteratorOfHolds () ;
               while ( !(fujaba__Success) && fujaba__IterP1HoldsR1.hasNext () )
               {
                  try
                  {
                     r1 = (Resource) fujaba__IterP1HoldsR1.next () ;
                      
                     // bind r2: Resource
                     fujaba__IterP1BlockedByR2 = p1.iteratorOfBlockedBy () ;
                     while ( !(fujaba__Success) && fujaba__IterP1BlockedByR2.hasNext () )
                     {
                        try
                        {
                           r2 = (Resource) fujaba__IterP1BlockedByR2.next () ;
                            
                           // check isomorphic binding
                           JavaSDM.ensure ( !(r1.equals (r2)) ) ;
                            
                           // bind p2: Process
                           fujaba__IterR1RequestedByP2 = r1.iteratorOfRequestedBy () ;
                           while ( !(fujaba__Success) && fujaba__IterR1RequestedByP2.hasNext () )
                           {
                              try
                              {
                                 p2 = (Process) fujaba__IterR1RequestedByP2.next () ;
                                  
                                 // check isomorphic binding
                                 JavaSDM.ensure ( !(p1.equals (p2)) ) ;
                                  
if(logger.isDebugEnabled()) {
	logger.debug("waitingRule: pm: " + System.nanoTime());
}
                                 //  delete link
                                 r2.removeFromBlocks (p1);
                                  
                                 // create link
                                 r2.addToBlocks (p2);
                                  
                                 fujaba__Success = true ;
                              }
                              catch ( JavaSDMException fujaba__InternalException )
                              {
                              }
                           }
                        }
                        catch ( JavaSDMException fujaba__InternalException )
                        {
                        }
                     }
                  }
                  catch ( JavaSDMException fujaba__InternalException )
                  {
                  }
               }
            }
            catch ( JavaSDMException fujaba__InternalException )
            {
            }
         }
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void removeYou()
   {
      removeAllFromProcesses ();
      removeAllFromResources ();
   }

}
