package one;

import de.upb.tools.fca.*;
import java.util.*;

public class Process
{
   /**
    * <pre>
    *          0..n   blocked   0..n 
    * Process ----------------------- Resource
    *          blocks      blockedBy 
    * </pre>
    */
   private FHashSet blockedBy;

   public boolean addToBlockedBy(Resource value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.blockedBy == null)
         {
            this.blockedBy = new FHashSet (); // or FTreeSet () or FLinkedList ()
         }
         
         changed = this.blockedBy.add (value);
         if (changed)
         {
            value.addToBlocks (this);
         }
         
      }
      return changed;
   }

   public boolean hasInBlockedBy(Resource value)
   {
      return ((this.blockedBy != null) &&
              (value != null) &&
              this.blockedBy.contains (value));
   }

   public Iterator iteratorOfBlockedBy()
   {
      return ((this.blockedBy == null)
              ? FEmptyIterator.get ()
              : this.blockedBy.iterator ());
   }

   public void removeAllFromBlockedBy()
   {
      Resource tmpValue;
      Iterator iter = this.iteratorOfBlockedBy ();
      while (iter.hasNext ())
      {
         tmpValue = (Resource) iter.next ();
         this.removeFromBlockedBy (tmpValue);
      }
   }

   public boolean removeFromBlockedBy(Resource value)
   {
      boolean changed = false;
      if ((this.blockedBy != null) && (value != null))
      {
         
         changed = this.blockedBy.remove (value);
         if (changed)
         {
            value.removeFromBlocks (this);
         }
         
      }
      return changed;
   }

   public int sizeOfBlockedBy()
   {
      return ((this.blockedBy == null)
              ? 0
              : this.blockedBy.size ());
   }

   /**
    * <pre>
    *          0..n   processContainer   0..1 
    * Process -------------------------------- Container
    *          processes            container 
    * </pre>
    */
   private Container container;

   public Container getContainer()
   {
      return this.container;
   }

   public boolean setContainer(Container value)
   {
      boolean changed = false;
      if (this.container != value)
      {
         Container oldValue = this.container;
         
         if (this.container != null)
         {
            this.container = null;
            oldValue.removeFromProcesses (this);
         }
         this.container = value;
         if (value != null)
         {
            value.addToProcesses (this);
         }
         changed = true;
         
      }
      return changed;
   }

   /**
    * <pre>
    *          0..1   held_by   0..n 
    * Process ----------------------- Resource
    *          heldBy          holds 
    * </pre>
    */
   private FHashSet holds;

   public boolean addToHolds(Resource value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.holds == null)
         {
            this.holds = new FHashSet (); // or FTreeSet () or FLinkedList ()
         }
         
         changed = this.holds.add (value);
         if (changed)
         {
            value.setHeldBy (this);
         }
         
      }
      return changed;
   }

   public boolean hasInHolds(Resource value)
   {
      return ((this.holds != null) &&
              (value != null) &&
              this.holds.contains (value));
   }

   public Iterator iteratorOfHolds()
   {
      return ((this.holds == null)
              ? FEmptyIterator.get ()
              : this.holds.iterator ());
   }

   public void removeAllFromHolds()
   {
      Resource tmpValue;
      Iterator iter = this.iteratorOfHolds ();
      while (iter.hasNext ())
      {
         tmpValue = (Resource) iter.next ();
         this.removeFromHolds (tmpValue);
      }
   }

   public boolean removeFromHolds(Resource value)
   {
      boolean changed = false;
      if ((this.holds != null) && (value != null))
      {
         
         changed = this.holds.remove (value);
         if (changed)
         {
            value.setHeldBy (null);
         }
         
      }
      return changed;
   }

   public int sizeOfHolds()
   {
      return ((this.holds == null)
              ? 0
              : this.holds.size ());
   }

   /**
    * <pre>
    *          0..1   next   0..1 
    * Process -------------------- Process
    *          previous      next 
    * </pre>
    */
   private Process next;

   public Process getNext()
   {
      return this.next;
   }

   public boolean setNext(Process value)
   {
      boolean changed = false;
      if (this.next != value)
      {
         Process oldValue = this.next;
         
         if (this.next != null)
         {
            this.next = null;
            oldValue.setPrevious (null);
         }
         this.next = value;
         if (value != null)
         {
            value.setPrevious (this);
         }
         changed = true;
         
      }
      return changed;
   }

   /**
    * <pre>
    *          0..1   next   0..1 
    * Process -------------------- Process
    *          next      previous 
    * </pre>
    */
   private Process previous;

   public Process getPrevious()
   {
      return this.previous;
   }

   public boolean setPrevious(Process value)
   {
      boolean changed = false;
      if (this.previous != value)
      {
         Process oldValue = this.previous;
         
         if (this.previous != null)
         {
            this.previous = null;
            oldValue.setNext (null);
         }
         this.previous = value;
         if (value != null)
         {
            value.setNext (this);
         }
         changed = true;
         
      }
      return changed;
   }

   /**
    * <pre>
    *          0..1    release     0..n 
    * Process -------------------------- Resource
    *          releasedBy      releases 
    * </pre>
    */
   private FHashSet releases;

   public boolean addToReleases(Resource value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.releases == null)
         {
            this.releases = new FHashSet (); // or FTreeSet () or FLinkedList ()
         }
         
         changed = this.releases.add (value);
         if (changed)
         {
            value.setReleasedBy (this);
         }
         
      }
      return changed;
   }

   public boolean hasInReleases(Resource value)
   {
      return ((this.releases != null) &&
              (value != null) &&
              this.releases.contains (value));
   }

   public Iterator iteratorOfReleases()
   {
      return ((this.releases == null)
              ? FEmptyIterator.get ()
              : this.releases.iterator ());
   }

   public void removeAllFromReleases()
   {
      Resource tmpValue;
      Iterator iter = this.iteratorOfReleases ();
      while (iter.hasNext ())
      {
         tmpValue = (Resource) iter.next ();
         this.removeFromReleases (tmpValue);
      }
   }

   public boolean removeFromReleases(Resource value)
   {
      boolean changed = false;
      if ((this.releases != null) && (value != null))
      {
         
         changed = this.releases.remove (value);
         if (changed)
         {
            value.setReleasedBy (null);
         }
         
      }
      return changed;
   }

   public int sizeOfReleases()
   {
      return ((this.releases == null)
              ? 0
              : this.releases.size ());
   }

   /**
    * <pre>
    *          0..n     request     0..n 
    * Process --------------------------- Resource
    *          requestedBy      requests 
    * </pre>
    */
   private FHashSet requests;

   public boolean addToRequests(Resource value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.requests == null)
         {
            this.requests = new FHashSet (); // or FTreeSet () or FLinkedList ()
         }
         
         changed = this.requests.add (value);
         if (changed)
         {
            value.addToRequestedBy (this);
         }
         
      }
      return changed;
   }

   public boolean hasInRequests(Resource value)
   {
      return ((this.requests != null) &&
              (value != null) &&
              this.requests.contains (value));
   }

   public Iterator iteratorOfRequests()
   {
      return ((this.requests == null)
              ? FEmptyIterator.get ()
              : this.requests.iterator ());
   }

   public void removeAllFromRequests()
   {
      Resource tmpValue;
      Iterator iter = this.iteratorOfRequests ();
      while (iter.hasNext ())
      {
         tmpValue = (Resource) iter.next ();
         this.removeFromRequests (tmpValue);
      }
   }

   public boolean removeFromRequests(Resource value)
   {
      boolean changed = false;
      if ((this.requests != null) && (value != null))
      {
         
         changed = this.requests.remove (value);
         if (changed)
         {
            value.removeFromRequestedBy (this);
         }
         
      }
      return changed;
   }

   public int sizeOfRequests()
   {
      return ((this.requests == null)
              ? 0
              : this.requests.size ());
   }

   /**
    * <pre>
    *          0..1   token   0..n 
    * Process --------------------- Resource
    *          token        tokens 
    * </pre>
    */
   private FHashSet tokens;

   public boolean addToTokens(Resource value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.tokens == null)
         {
            this.tokens = new FHashSet (); // or FTreeSet () or FLinkedList ()
         }
         
         changed = this.tokens.add (value);
         if (changed)
         {
            value.setToken (this);
         }
         
      }
      return changed;
   }

   public boolean hasInTokens(Resource value)
   {
      return ((this.tokens != null) &&
              (value != null) &&
              this.tokens.contains (value));
   }

   public Iterator iteratorOfTokens()
   {
      return ((this.tokens == null)
              ? FEmptyIterator.get ()
              : this.tokens.iterator ());
   }

   public void removeAllFromTokens()
   {
      Resource tmpValue;
      Iterator iter = this.iteratorOfTokens ();
      while (iter.hasNext ())
      {
         tmpValue = (Resource) iter.next ();
         this.removeFromTokens (tmpValue);
      }
   }

   public boolean removeFromTokens(Resource value)
   {
      boolean changed = false;
      if ((this.tokens != null) && (value != null))
      {
         
         changed = this.tokens.remove (value);
         if (changed)
         {
            value.setToken (null);
         }
         
      }
      return changed;
   }

   public int sizeOfTokens()
   {
      return ((this.tokens == null)
              ? 0
              : this.tokens.size ());
   }

   public void removeYou()
   {
      Process tmpNext = getNext ();
      if (tmpNext != null)
      {
         setNext (null);
      }
      Process tmpPrevious = getPrevious ();
      if (tmpPrevious != null)
      {
         setPrevious (null);
      }
      Container tmpContainer = getContainer ();
      if (tmpContainer != null)
      {
         setContainer (null);
      }
      removeAllFromRequests ();
      removeAllFromTokens ();
      removeAllFromHolds ();
      removeAllFromReleases ();
      removeAllFromBlockedBy ();
   }

}
