package one;

import de.upb.tools.sdm.*;
import java.util.*;
import de.upb.tools.fca.*;
import org.apache.log4j.Logger;

public class Resource
{
	private static Logger logger = Logger.getLogger(Resource.class);
   /**
    * <pre>
    *           0..n   blocked   0..n 
    * Resource ----------------------- Process
    *           blockedBy      blocks 
    * </pre>
    */
   private FHashSet blocks;

   public boolean addToBlocks(Process value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.blocks == null)
         {
            this.blocks = new FHashSet (); // or FTreeSet () or FLinkedList ()
         }
         
         changed = this.blocks.add (value);
         if (changed)
         {
            value.addToBlockedBy (this);
         }
         
      }
      return changed;
   }

   public boolean hasInBlocks(Process value)
   {
      return ((this.blocks != null) &&
              (value != null) &&
              this.blocks.contains (value));
   }

   public Iterator iteratorOfBlocks()
   {
      return ((this.blocks == null)
              ? FEmptyIterator.get ()
              : this.blocks.iterator ());
   }

   public void removeAllFromBlocks()
   {
      Process tmpValue;
      Iterator iter = this.iteratorOfBlocks ();
      while (iter.hasNext ())
      {
         tmpValue = (Process) iter.next ();
         this.removeFromBlocks (tmpValue);
      }
   }

   public boolean removeFromBlocks(Process value)
   {
      boolean changed = false;
      if ((this.blocks != null) && (value != null))
      {
         
         changed = this.blocks.remove (value);
         if (changed)
         {
            value.removeFromBlockedBy (this);
         }
         
      }
      return changed;
   }

   public int sizeOfBlocks()
   {
      return ((this.blocks == null)
              ? 0
              : this.blocks.size ());
   }

   /**
    * <pre>
    *           0..n   resourceContainer   0..1 
    * Resource --------------------------------- Container
    *           resources             container 
    * </pre>
    */
   private Container container;

   public Container getContainer()
   {
      return this.container;
   }

   public boolean setContainer(Container value)
   {
      boolean changed = false;
      if (this.container != value)
      {
         Container oldValue = this.container;
         
         if (this.container != null)
         {
            this.container = null;
            oldValue.removeFromResources (this);
         }
         this.container = value;
         if (value != null)
         {
            value.addToResources (this);
         }
         changed = true;
         
      }
      return changed;
   }

   /**
    * <pre>
    *           0..n   held_by   0..1 
    * Resource ----------------------- Process
    *           holds          heldBy 
    * </pre>
    */
   private Process heldBy;

   public Process getHeldBy()
   {
      return this.heldBy;
   }

   public boolean setHeldBy(Process value)
   {
      boolean changed = false;
      if (this.heldBy != value)
      {
         Process oldValue = this.heldBy;
         
         if (this.heldBy != null)
         {
            this.heldBy = null;
            oldValue.removeFromHolds (this);
         }
         this.heldBy = value;
         if (value != null)
         {
            value.addToHolds (this);
         }
         changed = true;
         
      }
      return changed;
   }

   /**
    * <pre>
    *           0..n    release     0..1 
    * Resource -------------------------- Process
    *           releases      releasedBy 
    * </pre>
    */
   private Process releasedBy;

   public Process getReleasedBy()
   {
      return this.releasedBy;
   }

   public boolean setReleasedBy(Process value)
   {
      boolean changed = false;
      if (this.releasedBy != value)
      {
         Process oldValue = this.releasedBy;
         
         if (this.releasedBy != null)
         {
            this.releasedBy = null;
            oldValue.removeFromReleases (this);
         }
         this.releasedBy = value;
         if (value != null)
         {
            value.addToReleases (this);
         }
         changed = true;
         
      }
      return changed;
   }

   /**
    * <pre>
    *           0..n     request     0..n 
    * Resource --------------------------- Process
    *           requests      requestedBy 
    * </pre>
    */
   private FHashSet requestedBy;

   public boolean addToRequestedBy(Process value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.requestedBy == null)
         {
            this.requestedBy = new FHashSet (); // or FTreeSet () or FLinkedList ()
         }
         
         changed = this.requestedBy.add (value);
         if (changed)
         {
            value.addToRequests (this);
         }
         
      }
      return changed;
   }

   public boolean hasInRequestedBy(Process value)
   {
      return ((this.requestedBy != null) &&
              (value != null) &&
              this.requestedBy.contains (value));
   }

   public Iterator iteratorOfRequestedBy()
   {
      return ((this.requestedBy == null)
              ? FEmptyIterator.get ()
              : this.requestedBy.iterator ());
   }

   public void removeAllFromRequestedBy()
   {
      Process tmpValue;
      Iterator iter = this.iteratorOfRequestedBy ();
      while (iter.hasNext ())
      {
         tmpValue = (Process) iter.next ();
         this.removeFromRequestedBy (tmpValue);
      }
   }

   public boolean removeFromRequestedBy(Process value)
   {
      boolean changed = false;
      if ((this.requestedBy != null) && (value != null))
      {
         
         changed = this.requestedBy.remove (value);
         if (changed)
         {
            value.removeFromRequests (this);
         }
         
      }
      return changed;
   }

   public int sizeOfRequestedBy()
   {
      return ((this.requestedBy == null)
              ? 0
              : this.requestedBy.size ());
   }

   /**
    * <pre>
    *           0..n   token   0..1 
    * Resource --------------------- Process
    *           tokens        token 
    * </pre>
    */
   private Process token;

   public Process getToken()
   {
      return this.token;
   }

   public boolean setToken(Process value)
   {
      boolean changed = false;
      if (this.token != value)
      {
         Process oldValue = this.token;
         
         if (this.token != null)
         {
            this.token = null;
            oldValue.removeFromTokens (this);
         }
         this.token = value;
         if (value != null)
         {
            value.addToTokens (this);
         }
         changed = true;
         
      }
      return changed;
   }

   public void giveRulePP()
   {
      boolean fujaba__Success = false ;
      Process p1 = null ;
      Process p2 = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p1: Process
         p1 = this.getReleasedBy () ;
         JavaSDM.ensure ( p1 != null ) ;
          
         // bind p2: Process
         p2 = p1.getNext () ;
         JavaSDM.ensure ( p2 != null ) ;
          
         // check isomorphic binding
         JavaSDM.ensure ( !(p1.equals (p2)) ) ;
          
if(logger.isDebugEnabled()) {
	logger.debug("giveRulePP: pm: " + System.nanoTime());
}
         //  delete link
         this.setReleasedBy (null);
          
         // create link
         this.setToken (p2);
          
         fujaba__Success = true ;
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void releaseRulePP()
   {
      boolean fujaba__Success = false ;
      Iterator fujaba__IterPRequestsRn = null ;
      Process p = null ;
      Resource rn = null ;
       
      try
      {
         fujaba__Success = false ;
          
         // bind p: Process
         p = this.getHeldBy () ;
         JavaSDM.ensure ( p != null ) ;
          
         // check negative application condition
         try
         {
            fujaba__Success = false ;
             
            // bind rn: Resource
            fujaba__IterPRequestsRn = p.iteratorOfRequests () ;
            while ( !(fujaba__Success) && fujaba__IterPRequestsRn.hasNext () )
            {
               try
               {
                  rn = (Resource) fujaba__IterPRequestsRn.next () ;
                   
                  // check isomorphic binding
                  JavaSDM.ensure ( !(this.equals (rn)) ) ;
                   
                  fujaba__Success = true ;
               }
               catch ( JavaSDMException fujaba__InternalException )
               {
               }
            }
         }
         catch ( JavaSDMException fujaba__InternalException )
         {
            fujaba__Success = false ;
         }
          
         fujaba__Success = !(fujaba__Success);
         JavaSDM.ensure ( fujaba__Success ) ;
         fujaba__Success = false;
         // end of negative application condition
          
if(logger.isDebugEnabled()) {
	logger.debug("releaseRulePP: pm: " + System.nanoTime());
}
         //  delete link
         this.setHeldBy (null);
          
         // create link
         this.setReleasedBy (p);
          
         fujaba__Success = true ;
      }
      catch ( JavaSDMException fujaba__InternalException )
      {
         fujaba__Success = false ;
      }
       
      return;
   }


   public void removeYou()
   {
      Container tmpContainer = getContainer ();
      if (tmpContainer != null)
      {
         setContainer (null);
      }
      removeAllFromRequestedBy ();
      Process tmpToken = getToken ();
      if (tmpToken != null)
      {
         setToken (null);
      }
      Process tmpHeldBy = getHeldBy ();
      if (tmpHeldBy != null)
      {
         setHeldBy (null);
      }
      Process tmpReleasedBy = getReleasedBy ();
      if (tmpReleasedBy != null)
      {
         setReleasedBy (null);
      }
      removeAllFromBlocks ();
   }

}
